@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2005  Bastian Kleineidam
+# Copyright (C) 2000-2006 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@ Main function module for link checking.
 import sys
 if not hasattr(sys, 'version_info') or \
    sys.version_info < (2, 4, 0, 'final', 0):
-    raise SystemExit, "This program requires Python 2.4 or later."
+    raise SystemExit("This program requires Python 2.4 or later.")
 import os
 import re
 
@@ -46,18 +46,29 @@ lognames = {
     "thread": LOG_THREAD,
     "all": LOG,
     }
-lognamelist = ", ".join(["%r"%name for name in lognames.keys()])
+lognamelist = ", ".join(["%r"%name for name in lognames.iterkeys()])
 
 import linkcheck.log
 
 
-class LinkCheckerError (Exception):
+class LinkCheckerError (StandardError):
     """
     Exception to be raised on linkchecker-specific check errors.
     """
     pass
 
 
+def add_intern_pattern (url_data, config):
+    """
+    Add intern URL regex to config.
+    """
+    pat = url_data.get_intern_pattern()
+    if pat:
+        assert None == linkcheck.log.debug(LOG_CHECK,
+           "Add intern pattern %r from command line", pat)
+        config['internlinks'].append(get_link_pat(pat))
+
+
 def get_link_pat (arg, strict=False):
     """
     Get a link pattern matcher for intern/extern links.
@@ -70,7 +81,7 @@ def get_link_pat (arg, strict=False):
     @return: dictionary with keys 'pattern', 'negate' and 'strict'
     @rtype: dict
     """
-    linkcheck.log.debug(LOG_CHECK, "Link pattern %r", arg)
+    assert None == linkcheck.log.debug(LOG_CHECK, "Link pattern %r", arg)
     if arg.startswith('!'):
         pattern = arg[1:]
         negate = True
@@ -111,7 +122,7 @@ Loggers = {
     "none": linkcheck.logger.none.NoneLogger,
 }
 # for easy printing: a comma separated logger list
-LoggerKeys = ", ".join(["%r"%name for name in Loggers.keys()])
+LoggerKeys = ", ".join(["%r"%name for name in Loggers.iterkeys()])
 
 
 def init_i18n ():
