@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2005  Bastian Kleineidam
+# Copyright (C) 2000-2006 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -53,12 +53,12 @@ class FtpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         order: login, changing directory, list the file.
         """
         # proxy support (we support only http)
-        self.set_proxy(self.consumer.config("proxy").get(self.scheme))
+        self.set_proxy(self.aggregate.config["proxy"].get(self.scheme))
         if self.proxy:
             # using a (HTTP) proxy
             http = httpurl.HttpUrl(self.base_url,
                   self.recursion_level,
-                  self.consumer,
+                  self.aggregate,
                   parent_url=self.parent_url,
                   base_ref=self.base_ref,
                   line=self.line,
@@ -87,7 +87,7 @@ class FtpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         # ready to connect
         _user, _password = self.get_user_password()
         key = ("ftp", self.urlparts[1], _user, _password)
-        conn = self.consumer.get_connection(key)
+        conn = self.aggregate.connections.get(key)
         if conn is not None and conn.sock is not None:
             # reuse cached FTP connection
             self.url_connection = conn
@@ -105,12 +105,12 @@ class FtpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
                 self.url_connection.login(_user, _password)
         except EOFError, msg:
             msg = str(msg)
-            raise linkcheck.LinkCheckerError, \
-                   _("Remote host has closed connection: %r") % msg
+            raise linkcheck.LinkCheckerError(
+                   _("Remote host has closed connection: %r") % msg)
         if not self.url_connection.getwelcome():
             self.close_connection()
-            raise linkcheck.LinkCheckerError, \
-                   _("Got no answer from FTP server")
+            raise linkcheck.LinkCheckerError(
+                   _("Got no answer from FTP server"))
         # don't set info anymore, this may change every time we log in
         #self.add_info(linkcheck.strformat.unicode_safe(info))
 
@@ -134,7 +134,8 @@ class FtpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         if not self.filename:
             return
         files = self.get_files()
-        linkcheck.log.debug(linkcheck.LOG_CHECK, "FTP files %s", str(files))
+        assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+            "FTP files %s", str(files))
         if self.filename in files:
             # file found
             return
@@ -146,7 +147,7 @@ class FtpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
                          tag="ftp-missing-slash")
                 self.url += '/'
             return
-        raise ftplib.error_perm, "550 File not found"
+        raise ftplib.error_perm("550 File not found")
 
     def get_files (self):
         """
@@ -158,8 +159,8 @@ class FtpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
             """
             Parse list line and add the entry it points to to the file list.
             """
-            linkcheck.log.debug(linkcheck.LOG_CHECK, "Directory entry %r",
-                                line)
+            assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                "Directory entry %r", line)
             try:
                 fpo = ftpparse.parse(line)
                 name = fpo.name
@@ -168,8 +169,8 @@ class FtpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
                 if fpo.trycwd or fpo.tryretr:
                     files.append(name)
             except (ValueError, AttributeError), msg:
-                linkcheck.log.debug(linkcheck.LOG_CHECK, "%s (%s)",
-                                    str(msg), line)
+                assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                    "%s (%s)", str(msg), line)
         self.url_connection.dir(add_entry)
         return files
 
@@ -187,7 +188,7 @@ class FtpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         """
         if self.is_directory():
             return True
-        for ro in linkcheck.checker.extensions.values():
+        for ro in linkcheck.checker.extensions.itervalues():
             if ro.search(self.url):
                 return True
         return False
@@ -203,11 +204,11 @@ class FtpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         Parse URL target for links.
         """
         if self.is_directory():
-            return self.parse_html()
-        for key, ro in linkcheck.checker.extensions.items():
+            self.parse_html()
+            return
+        for key, ro in linkcheck.checker.extensions.iteritems():
             if ro.search(self.url):
-                return getattr(self, "parse_"+key)()
-        return None
+                getattr(self, "parse_"+key)()
 
     def get_content (self):
         """
@@ -247,6 +248,6 @@ class FtpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         # add to cached connections
         _user, _password = self.get_user_password()
         key = ("ftp", self.urlparts[1], _user, _password)
-        cache_add = self.consumer.add_connection
+        cache_add = self.aggregate.connections.add
         cache_add(key, self.url_connection, DEFAULT_TIMEOUT_SECS)
         self.url_connection = None
