@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2005  Bastian Kleineidam
+# Copyright (C) 2000-2006 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -63,7 +63,7 @@ class ProxySupport (object):
         """
         Check if self.host is in the no-proxy-for ignore list.
         """
-        for ro in self.consumer.config("noproxyfor"):
+        for ro in self.aggregate.config["noproxyfor"]:
             if ro.search(self.host):
                 return True
         return False
