@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2005  Bastian Kleineidam
+# Copyright (C) 2000-2006 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -32,6 +32,9 @@ import traceback
 import linkcheck
 import linkcheck.linkparse
 import linkcheck.checker
+import linkcheck.cache.geoip
+import linkcheck.director
+import linkcheck.director.status
 import linkcheck.strformat
 import linkcheck.containers
 import linkcheck.log
@@ -50,12 +53,23 @@ def urljoin (parent, url, scheme):
     return urlparse.urljoin(parent, url)
 
 
+def url_norm (url):
+    """
+    Wrapper for url.url_norm() to convert UnicodeError in LinkCheckerError.
+    """
+    try:
+        return linkcheck.url.url_norm(url)
+    except UnicodeError:
+        msg = _("URL has unparsable domain name: %s") % sys.exc_info()[1]
+        raise linkcheck.LinkCheckerError(msg)
+
+
 class UrlBase (object):
     """
     An URL with additional information like validity etc.
     """
 
-    def __init__ (self, base_url, recursion_level, consumer,
+    def __init__ (self, base_url, recursion_level, aggregate,
                   parent_url = None, base_ref = None,
                   line = -1, column = -1, name = u""):
         """
@@ -63,7 +77,7 @@ class UrlBase (object):
 
         @param base_url: unquoted and possibly unnormed url
         @param recursion_level: on what check level lies the base url
-        @param consumer: consumer instance
+        @param aggregate: aggregate instance
         @param parent_url: quoted and normed url of parent or None
         @param base_ref: quoted and normed url of <base href=""> or None
         @param line: line number of url in parent content
@@ -71,13 +85,13 @@ class UrlBase (object):
         @param name: name of url or empty
         """
         self.init(base_ref, base_url, parent_url, recursion_level,
-                  consumer, line, column, name)
+                  aggregate, line, column, name)
         self.reset()
         self.check_syntax()
 
 
     def init (self, base_ref, base_url, parent_url, recursion_level,
-              consumer, line, column, name):
+              aggregate, line, column, name):
         """
         Initialize internal data.
         """
@@ -86,7 +100,7 @@ class UrlBase (object):
         self.base_url = base_url
         self.parent_url = parent_url
         self.recursion_level = recursion_level
-        self.consumer = consumer
+        self.aggregate = aggregate
         self.line = line
         self.column = column
         self.name = name
@@ -192,17 +206,18 @@ class UrlBase (object):
         """
         self.warnings.append((tag, s))
 
-    def add_info (self, s):
+    def add_info (self, s, tag=None):
         """
         Add an info string.
         """
-        self.info.append(s)
+        self.info.append((tag, s))
 
     def copy_from_cache (self, cache_data):
         """
         Fill attributes from cache data.
         """
         self.result = cache_data["result"]
+        self.has_result = True
         self.warnings.extend(cache_data["warnings"])
         self.info.extend(cache_data["info"])
         self.valid = cache_data["valid"]
@@ -222,6 +237,13 @@ class UrlBase (object):
                 "dlsize": self.dlsize,
                }
 
+    def get_alias_cache_data (self):
+        """
+        Return all data values that should be put in the cache.
+        Intended to be overridden by subclasses that handle aliases.
+        """
+        return self.get_cache_data()
+
     def set_cache_keys (self):
         """
         Set keys for URL checking and content recursion.
@@ -230,11 +252,11 @@ class UrlBase (object):
         # URLs with different anchors to have the same content
         self.cache_content_key = urlparse.urlunsplit(self.urlparts[:4]+[u''])
         assert isinstance(self.cache_content_key, unicode), self
-        linkcheck.log.debug(linkcheck.LOG_CACHE, "Content cache key %r",
-                            self.cache_content_key)
+        assert None == linkcheck.log.debug(linkcheck.LOG_CACHE,
+            "Content cache key %r", self.cache_content_key)
         # construct cache key
-        if self.consumer.config("anchorcaching") and \
-           self.consumer.config("anchors"):
+        if self.aggregate.config["anchorcaching"] and \
+           self.aggregate.config["anchors"]:
             # do not ignore anchor
             parts = self.urlparts[:]
             parts[4] = self.anchor
@@ -243,8 +265,8 @@ class UrlBase (object):
             # no anchor caching
             self.cache_url_key = self.cache_content_key
         assert isinstance(self.cache_url_key, unicode), self
-        linkcheck.log.debug(linkcheck.LOG_CACHE, "URL cache key %r",
-                            self.cache_url_key)
+        assert None == linkcheck.log.debug(linkcheck.LOG_CACHE,
+            "URL cache key %r", self.cache_url_key)
 
     def check_syntax (self):
         """
@@ -256,9 +278,12 @@ class UrlBase (object):
         @return: True if syntax is correct, else False.
         @rtype: bool
         """
-        linkcheck.log.debug(linkcheck.LOG_CHECK, "checking syntax")
-        if (self.base_url is None) or \
-           (not self.base_url and not self.parent_url):
+        assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+            "checking syntax")
+        if self.base_url is None:
+            self.set_result(_("URL is missing"), valid=False)
+            return
+        if not (self.base_url or self.parent_url):
             self.set_result(_("URL is empty"), valid=False)
             return
         try:
@@ -280,8 +305,8 @@ class UrlBase (object):
         Construct self.url and self.urlparts out of the given base
         url information self.base_url, self.parent_url and self.base_ref.
         """
-        # norm base url
-        base_url, is_idn = linkcheck.url.url_norm(self.base_url)
+        # norm base url - can raise UnicodeError from url.idna_encode()
+        base_url, is_idn = url_norm(self.base_url)
         if is_idn:
             self.add_warning(_("""URL %r has a unicode domain name which
                           is not yet widely supported. You should use
@@ -289,7 +314,7 @@ class UrlBase (object):
                           tag="url-unicode-domain")
         elif self.base_url != base_url:
             self.add_warning(
-              _("Base URL is not properly normed. Normed URL is %(url)s.") % \
+              _("Base URL is not properly normed. Normed URL is %(url)s.") %
                {'url': base_url}, tag="url-unnormed")
         # make url absolute
         if self.base_ref:
@@ -328,40 +353,36 @@ class UrlBase (object):
         self.host, self.port = urllib.splitport(host)
         if self.port is not None:
             if not linkcheck.url.is_numeric_port(self.port):
-                raise linkcheck.LinkCheckerError, \
-                         _("URL has invalid port %r") % str(self.port)
+                raise linkcheck.LinkCheckerError(
+                         _("URL has invalid port %r") % str(self.port))
             self.port = int(self.port)
 
     def check (self):
         """
         Main check function for checking this URL.
         """
-        if self.consumer.config("trace"):
-            linkcheck.log.trace()
+        if self.aggregate.config["trace"]:
+            linkcheck.trace.trace_on()
         try:
             self.local_check()
-            self.consumer.checked(self)
         except (socket.error, select.error):
-            self.consumer.interrupted(self)
             # on Unix, ctrl-c can raise
             # error: (4, 'Interrupted system call')
             etype, value = sys.exc_info()[:2]
-            if etype == 4:
-                raise KeyboardInterrupt, value
+            if etype == errno.EINTR:
+                raise KeyboardInterrupt(value)
             else:
                 raise
         except KeyboardInterrupt:
-            self.consumer.interrupted(self)
             raise
         except:
-            self.consumer.interrupted(self)
-            linkcheck.checker.internal_error()
+            linkcheck.director.console.internal_error()
 
     def add_country_info (self):
         """
         Try to ask GeoIP database for country info.
         """
-        country = self.consumer.get_country_name(self.host)
+        country = linkcheck.cache.geoip.get_country(self.host)
         if country is not None:
             self.add_info(_("URL is located in %s.") % _(country))
 
@@ -369,24 +390,22 @@ class UrlBase (object):
         """
         Local check function can be overridden in subclasses.
         """
-        linkcheck.log.debug(linkcheck.LOG_CHECK, "Checking %s", self)
-        if self.recursion_level and self.consumer.config('wait'):
-            linkcheck.log.debug(linkcheck.LOG_CHECK,
-                                "sleeping for %d seconds",
-                                self.consumer.config('wait'))
-            time.sleep(self.consumer.config('wait'))
-        t = time.time()
+        assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+            "Checking %s", self)
+        # start time for check
+        check_start = time.time()
         self.set_extern(self.url)
         if self.extern[0] and self.extern[1]:
             self.add_info(_("Outside of domain filter, checked only syntax."))
             return
 
         # check connection
-        linkcheck.log.debug(linkcheck.LOG_CHECK, "checking connection")
+        assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+            "checking connection")
         try:
             self.check_connection()
             self.add_country_info()
-            if self.consumer.config("anchors"):
+            if self.aggregate.config["anchors"]:
                 self.check_anchors()
         except tuple(linkcheck.checker.ExcList):
             value = self.handle_exception()
@@ -400,9 +419,10 @@ class UrlBase (object):
                             valid=False)
 
         # check content
-        warningregex = self.consumer.config("warningregex")
+        warningregex = self.aggregate.config["warningregex"]
         if warningregex and self.valid:
-            linkcheck.log.debug(linkcheck.LOG_CHECK, "checking content")
+            assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                "checking content")
             try:
                 self.check_content(warningregex)
             except tuple(linkcheck.checker.ExcList):
@@ -410,7 +430,7 @@ class UrlBase (object):
                 self.set_result(linkcheck.strformat.unicode_safe(value),
                                 valid=False)
 
-        self.checktime = time.time() - t
+        self.checktime = time.time() - check_start
         # check recursion
         try:
             if self.allows_recursion():
@@ -443,8 +463,8 @@ class UrlBase (object):
         An exception occurred. Log it and set the cache flag.
         """
         etype, value, tb = sys.exc_info()
-        linkcheck.log.debug(linkcheck.LOG_CHECK, "exception %s",
-                            traceback.format_tb(tb))
+        assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+            "exception %s", traceback.format_tb(tb))
         # note: etype must be the exact class, not a subclass
         if (etype in linkcheck.checker.ExcNoCacheList) or \
            (etype == socket.error and value[0]==errno.EBADF) or \
@@ -464,30 +484,36 @@ class UrlBase (object):
         """
         Return True iff we can recurse into the url's content.
         """
-        linkcheck.log.debug(linkcheck.LOG_CHECK, "checking recursion...")
+        assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+            "checking recursion of %r ...", self.url)
         # Test self.valid before self.is_parseable().
         if not self.valid:
-            linkcheck.log.debug(linkcheck.LOG_CHECK, "... no, invalid.")
+            assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                "... no, invalid.")
             return False
         if not self.is_parseable():
-            linkcheck.log.debug(linkcheck.LOG_CHECK, "... no, not parseable.")
+            assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                ".. no, not parseable.")
             return False
         if not self.can_get_content():
-            linkcheck.log.debug(linkcheck.LOG_CHECK,
-                                "... no, cannot get content.")
+            assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                "... no, cannot get content.")
             return False
-        if self.consumer.config("recursionlevel") >= 0 and \
-           self.recursion_level >= self.consumer.config("recursionlevel"):
-            linkcheck.log.debug(linkcheck.LOG_CHECK,
-                                "... no, maximum recursion level reached.")
+        rec_level = self.aggregate.config["recursionlevel"]
+        if  rec_level >= 0 and self.recursion_level >= rec_level:
+            assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                "... no, maximum recursion level reached.")
             return False
         if self.extern[0]:
-            linkcheck.log.debug(linkcheck.LOG_CHECK, "... no, extern.")
+            assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                "... no, extern.")
             return False
         if not self.content_allows_robots():
-            linkcheck.log.debug(linkcheck.LOG_CHECK, "... no, robots.")
+            assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                "... no, robots.")
             return False
-        linkcheck.log.debug(linkcheck.LOG_CHECK, "... yes.")
+        assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+            "... yes, recursion.")
         return True
 
     def content_allows_robots (self):
@@ -517,8 +543,8 @@ class UrlBase (object):
                 self.can_get_content()):
             # do not bother
             return
-        linkcheck.log.debug(linkcheck.LOG_CHECK, "checking anchor %r",
-                            self.anchor)
+        assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+            "checking anchor %r", self.anchor)
         h = linkcheck.linkparse.LinkFinder(self.get_content(),
                                    tags={'a': [u'name'], None: [u'id']})
         p = linkcheck.HtmlParser.htmlsax.parser(h)
@@ -541,22 +567,24 @@ class UrlBase (object):
 
         @return: None
         """
-        for entry in self.consumer.config("externlinks"):
+        for entry in self.aggregate.config["externlinks"]:
             match = entry['pattern'].search(url)
             if (entry['negate'] and not match) or \
                (match and not entry['negate']):
-                linkcheck.log.debug(linkcheck.LOG_CHECK, "Extern URL %r", url)
+                assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                    "Extern URL %r", url)
                 self.extern = (1, entry['strict'])
                 return
-        for entry in self.consumer.config("internlinks"):
+        for entry in self.aggregate.config["internlinks"]:
             match = entry['pattern'].search(url)
             if (entry['negate'] and not match) or \
                (match and not entry['negate']):
-                linkcheck.log.debug(linkcheck.LOG_CHECK, "Intern URL %r", url)
+                assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                    "Intern URL %r", url)
                 self.extern = (0, 0)
                 return
-        linkcheck.log.debug(linkcheck.LOG_CHECK,
-                            "Explicit extern URL %r", url)
+        assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+            "Explicit extern URL %r", url)
         self.extern = (1, 0)
         return
 
@@ -595,9 +623,9 @@ class UrlBase (object):
         If a maximum size was given, call this function to check it
         against the content size of this url.
         """
-        maxbytes = self.consumer.config("warnsizebytes")
+        maxbytes = self.aggregate.config["warnsizebytes"]
         if maxbytes is not None and self.dlsize >= maxbytes:
-            self.add_warning(_("Content size %s is larger than %s.") % \
+            self.add_warning(_("Content size %s is larger than %s.") %
                          (linkcheck.strformat.strsize(self.dlsize),
                           linkcheck.strformat.strsize(maxbytes)),
                           tag="url-content-too-large")
@@ -607,8 +635,6 @@ class UrlBase (object):
         Parse url content and search for recursive links.
         Default parse type is html.
         """
-        linkcheck.log.debug(linkcheck.LOG_CHECK,
-                            "Parsing recursively into %s", self)
         self.parse_html()
 
     def get_user_password (self):
@@ -616,7 +642,7 @@ class UrlBase (object):
         Get tuple (user, password) from configured authentication.
         Both user and password can be None if not specified.
         """
-        for auth in self.consumer.config("authentication"):
+        for auth in self.aggregate.config["authentication"]:
             if auth['pattern'].match(self.url):
                 return auth['user'], auth['password']
         return None, None
@@ -626,6 +652,8 @@ class UrlBase (object):
         Parse into HTML content and search for URLs to check.
         Found URLs are added to the URL queue.
         """
+        assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+            "Parsing HTML %s", self)
         h = linkcheck.linkparse.LinkFinder(self.get_content())
         p = linkcheck.HtmlParser.htmlsax.parser(h)
         h.parser = p
@@ -633,24 +661,23 @@ class UrlBase (object):
         p.flush()
         h.parser = None
         p.handler = None
-        for s in h.parse_info:
-            # the parser had warnings/errors
-            self.add_warning(s)
         for url, line, column, name, codebase in h.urls:
             if codebase:
                 base_ref = codebase
             else:
                 base_ref = h.base_ref
             url_data = linkcheck.checker.get_url_from(url,
-                  self.recursion_level+1, self.consumer, parent_url=self.url,
+                  self.recursion_level+1, self.aggregate, parent_url=self.url,
                   base_ref=base_ref, line=line, column=column, name=name,
-                  cmdline=False)
-            self.consumer.append_url(url_data)
+                  assume_local=False)
+            self.aggregate.urlqueue.put(url_data)
 
     def parse_opera (self):
         """
         Parse an opera bookmark file.
         """
+        assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+            "Parsing Opera bookmarks %s", self)
         name = ""
         lineno = 0
         lines = self.get_content().splitlines()
@@ -663,10 +690,10 @@ class UrlBase (object):
                 url = line[4:]
                 if url:
                     url_data = linkcheck.checker.get_url_from(url,
-                              self.recursion_level+1, self.consumer,
+                              self.recursion_level+1, self.aggregate,
                               parent_url=self.url, line=lineno, name=name,
-                              cmdline=False)
-                    self.consumer.append_url(url_data)
+                              assume_local=False)
+                    self.aggregate.urlqueue.put(url_data)
                 name = ""
 
     def parse_text (self):
@@ -674,6 +701,8 @@ class UrlBase (object):
         Parse a text file with on url per line; comment and blank
         lines are ignored.
         """
+        assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+            "Parsing text %s", self)
         lineno = 0
         for line in self.get_content().splitlines():
             lineno += 1
@@ -681,15 +710,17 @@ class UrlBase (object):
             if not line or line.startswith('#'):
                 continue
             url_data = linkcheck.checker.get_url_from(line,
-                              self.recursion_level+1, self.consumer,
+                              self.recursion_level+1, self.aggregate,
                               parent_url=self.url, line=lineno,
-                              cmdline=False)
-            self.consumer.append_url(url_data)
+                              assume_local=False)
+            self.aggregate.urlqueue.put(url_data)
 
     def parse_css (self):
         """
         Parse a CSS file for url() patterns.
         """
+        assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+            "Parsing CSS %s", self)
         lineno = 0
         for line in self.get_content().splitlines():
             lineno += 1
@@ -697,10 +728,10 @@ class UrlBase (object):
                 column = mo.start("url")
                 url = linkcheck.strformat.unquote(mo.group("url").strip())
                 url_data = linkcheck.checker.get_url_from(url,
-                             self.recursion_level+1, self.consumer,
+                             self.recursion_level+1, self.aggregate,
                              parent_url=self.url, line=lineno, column=column,
-                             cmdline=False)
-                self.consumer.append_url(url_data)
+                             assume_local=False)
+                self.aggregate.urlqueue.put(url_data)
 
     def serialized (self):
         """
@@ -743,7 +774,7 @@ class UrlBase (object):
         @rtype: string
         """
         s = self.serialized()
-        return self.consumer.config('logger').encode(s)
+        return self.aggregate.config['logger'].encode(s)
 
     def __repr__ (self):
         """
@@ -753,3 +784,4 @@ class UrlBase (object):
         @rtype: unicode
         """
         return u"<%s >" % self.serialized()
+
