@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2004-2005  Bastian Kleineidam
+# Copyright (C) 2004-2006 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,41 +25,33 @@ class SetList (list):
     """
 
     def append (self, item):
-        """
-        Append only if not already there.
-        """
+        """Append only if not already there."""
         if item not in self:
             super(SetList, self).append(item)
 
     def extend (self, itemlist):
-        """
-        Extend while eliminating duplicates by appending item for item.
-        """
+        """Extend while eliminating duplicates by appending item for item."""
         for item in itemlist:
             self.append(item)
 
     def insert (self, index, item):
-        """
-        Insert item at given index only if it is not already there.
-        """
+        """Insert item at given index only if it is not already there."""
         if item not in self:
             super(SetList, self).insert(index, item)
 
     def __setitem__ (self, index, item):
-        """
-        Set new value, and eliminate a possible duplicate value.
-        """
+        """Set new value, and eliminate a possible duplicate value."""
         # search index i with self[i] == item
         delidx = -1
-        for i in range(len(self)):
+        for i in xrange(len(self)):
             if self[i] == item and i != index:
                 delidx = i
-                # break here, there can be only one duplicate
+                # stop here, there can be only one duplicate
                 break
-        # insert new value
+        # set new value
         super(SetList, self).__setitem__(index, item)
+        # remove duplicate
         if delidx != -1:
-            # remove duplicate
             del self[delidx]
 
 
@@ -70,79 +62,117 @@ class ListDict (dict):
     """
 
     def __init__ (self):
-        """
-        Initialize sorted key list.
-        """
+        """Initialize sorted key list."""
         super(ListDict, self).__init__()
         # sorted list of keys
         self._keys = []
 
     def __setitem__ (self, key, value):
-        """
-        Add key,value to dict, append key to sorted list.
-        """
+        """Add key,value to dict, append key to sorted list."""
         if not self.has_key(key):
             self._keys.append(key)
         super(ListDict, self).__setitem__(key, value)
 
     def __delitem__ (self, key):
-        """
-        Remove key from dict.
-        """
+        """Remove key from dict."""
         self._keys.remove(key)
         super(ListDict, self).__delitem__(key)
 
     def values (self):
-        """
-        Return sorted list of values.
-        """
+        """Return sorted list of values."""
         return [self[k] for k in self._keys]
 
     def items (self):
-        """
-        Return sorted list of items.
-        """
+        """Return sorted list of items."""
         return [(k, self[k]) for k in self._keys]
 
     def keys (self):
-        """
-        Return sorted list of keys.
-        """
+        """Return sorted list of keys."""
         return self._keys[:]
 
     def itervalues (self):
-        """
-        Return iterator over sorted values.
-        """
+        """Return iterator over sorted values."""
         return iter(self.values())
 
     def iteritems (self):
-        """return iterator over sorted items"""
+        """Return iterator over sorted items."""
         return iter(self.items())
 
     def iterkeys (self):
-        """
-        Return iterator over sorted keys.
-        """
+        """Return iterator over sorted keys."""
         return iter(self.keys())
 
     def clear (self):
-        """
-        Remove all dict entries.
-        """
+        """Remove all dict entries."""
         self._keys = []
         super(ListDict, self).clear()
 
     def get_true (self, key, default):
         """
-        Reteurn default element if key is not in the dict, of if the
-        self[key] evaluates to False.
+        Return default element if key is not in the dict, or if self[key]
+        evaluates to False. Useful for example if value is None, but
+        default value should be an empty string.
         """
         if key not in self or not self[key]:
             return default
         return self[key]
 
 
+class CaselessDict (dict):
+    """A dictionary ignoring the case of keys (which must be strings)."""
+
+    def __init__ (self):
+        dict.__init__(self)
+
+    def __getitem__ (self, key):
+        assert isinstance(key, basestring)
+        return dict.__getitem__(self, key.lower())
+
+    def __setitem__ (self, key, value):
+        assert isinstance(key, basestring)
+        dict.__setitem__(self, key.lower(), value)
+
+    def __contains__ (self, key):
+        assert isinstance(key, basestring)
+        return dict.__contains__(self, key.lower())
+
+    def has_key (self, key):
+        assert isinstance(key, basestring)
+        return dict.has_key(self, key.lower())
+
+    def get (self, key, def_val=None):
+        assert isinstance(key, basestring)
+        return dict.get(self, key.lower(), def_val)
+
+    def setdefault (self, key, def_val=None):
+        assert isinstance(key, basestring)
+        return dict.setdefault(self, key.lower(), def_val)
+
+    def update (self, other):
+        for k, v in other.items():
+            dict.__setitem__(self, k.lower(), v)
+
+    def fromkeys (self, iterable, value=None):
+        d = CaselessDict()
+        for k in iterable:
+            dict.__setitem__(d, k.lower(), value)
+        return d
+
+    def pop (self, key, def_val=None):
+        assert isinstance(key, basestring)
+        return dict.pop(self, key.lower(), def_val)
+
+
+class CaselessSortedDict (CaselessDict):
+    """Caseless dictionary with sorted keys."""
+
+    def keys (self):
+        return sorted(super(CaselessSortedDict, self).keys())
+
+    def items (self):
+        return [(x, self[x]) for x in self.keys()]
+
+
 class Node (object):
     """
     Internal node with pointers to sisters.
@@ -183,33 +213,25 @@ class LRU (object):
         self.first = None
         self.last = None
         if pairs is not None:
-            for key, value in pairs:
+            for key, value in pairs.items():
                 self[key] = value
 
     def __contains__ (self, obj):
-        """
-        Look if obj is in the queue.
-        """
+        """Look if obj is in the queue."""
         return obj in self.d
 
     def has_key (self, obj):
-        """
-        Look if obj is in the queue.
-        """
+        """Look if obj is in the queue."""
         return self.d.has_key(obj)
 
     def __getitem__ (self, obj):
-        """
-        Get stored object data, and mark it as LRU.
-        """
+        """Get stored object data, and mark it as LRU."""
         a = self.d[obj].me
         self[a[0]] = a[1]
         return a[1]
 
     def __setitem__ (self, obj, val):
-        """
-        Set given object data, and mark it as LRU.
-        """
+        """Set given object data, and mark it as LRU."""
         if obj in self.d:
             del self[obj]
         nobj = Node(self.last, (obj, val))
@@ -232,9 +254,7 @@ class LRU (object):
             del a
 
     def __delitem__ (self, obj):
-        """
-        Remove object from queue.
-        """
+        """Remove object from queue."""
         nobj = self.d[obj]
         if nobj.prev:
             nobj.prev.next = nobj.next
@@ -247,9 +267,7 @@ class LRU (object):
         del self.d[obj]
 
     def __iter__ (self):
-        """
-        Iterate over stored object values.
-        """
+        """Iterate over stored object values."""
         cur = self.first
         while cur != None:
             cur2 = cur.next
@@ -257,9 +275,7 @@ class LRU (object):
             cur = cur2
 
     def iteritems (self):
-        """
-        Iterate over stored object items.
-        """
+        """Iterate over stored object items."""
         cur = self.first
         while cur != None:
             cur2 = cur.next
@@ -267,22 +283,16 @@ class LRU (object):
             cur = cur2
 
     def iterkeys (self):
-        """
-        Iterate over stored object keys.
-        """
+        """Iterate over stored object keys."""
         return iter(self.d)
 
     def itervalues (self):
-        """
-        Iterate over stored object values.
-        """
+        """Iterate over stored object values."""
         for dummy, j in self.iteritems():
             yield j
 
     def keys (self):
-        """
-        Iterate over stored object keys.
-        """
+        """Iterate over stored object keys."""
         return self.d.keys()
 
     def setdefault (self, key, failobj=None):
