@@ -56,6 +56,17 @@ class NoNameservers(linkcheck.dns.exception.DNSException):
     """No non-broken nameservers are available to answer the query."""
     pass
 
+class NotAbsolute(linkcheck.dns.exception.DNSException):
+    """Raised if an absolute domain name is required but a relative name
+    was provided."""
+    pass
+
+class NoRootSOA(linkcheck.dns.exception.DNSException):
+    """Raised if for some reason there is no SOA at the root name.
+    This should never happen!"""
+    pass
+
+
 class Answer(object):
     """DNS stub resolver answer
 
@@ -289,6 +300,7 @@ class Resolver(object):
           'localhost',
           'loopback',
           '127.0.0.1',
+          '0.0.0.0',
           '::1',
           'ip6-localhost',
           'ip6-loopback',
@@ -340,6 +352,7 @@ class Resolver(object):
     def read_local_hosts (self):
         self.add_addrinfo(socket.gethostname())
         # add system specific hosts for all enabled interfaces
+        self.add_addrinfo('localhost', interface=True)
         for addr in self.read_local_ifaddrs():
             self.add_addrinfo(addr, interface=True)
 
@@ -572,14 +585,16 @@ class Resolver(object):
                 request.use_tsig(self.keyring, self.keyname)
             request.use_edns(self.edns, self.ednsflags, self.payload)
             response = None
-            #
+
             # make a copy of the servers list so we can alter it later.
-            #
+
             nameservers = self.nameservers[:]
             backoff = 0.10
             while response is None:
                 if len(nameservers) == 0:
-                    raise NoNameservers, "No DNS servers could answer the query"
+                    raise NoNameservers, \
+                      "No DNS servers %s could answer the query %s" % \
+                      (str(self.nameservers), str(qname))
                 for nameserver in nameservers:
                     timeout = self._compute_timeout(start)
                     try:
@@ -590,24 +605,24 @@ class Resolver(object):
                             response = linkcheck.dns.query.udp(request, nameserver,
                                                      timeout, self.port)
                     except (socket.error, linkcheck.dns.exception.Timeout):
-                        #
+
                         # Communication failure or timeout.  Go to the
                         # next server
-                        #
+
                         response = None
                         continue
                     except linkcheck.dns.query.UnexpectedSource:
-                        #
+
                         # Who knows?  Keep going.
-                        #
+
                         response = None
                         continue
                     except linkcheck.dns.exception.FormError:
-                        #
+
                         # We don't understand what this server is
                         # saying.  Take it out of the mix and
                         # continue.
-                        #
+
                         nameservers.remove(nameserver)
                         response = None
                         continue
@@ -615,22 +630,22 @@ class Resolver(object):
                     if rcode == linkcheck.dns.rcode.NOERROR or \
                            rcode == linkcheck.dns.rcode.NXDOMAIN:
                         break
-                    #
+
                     # We got a response, but we're not happy with the
                     # rcode in it.  Remove the server from the mix if
                     # the rcode isn't SERVFAIL.
-                    #
+
                     if rcode != linkcheck.dns.rcode.SERVFAIL:
                         nameservers.remove(nameserver)
                     response = None
-                #
+
                 # All nameservers failed!
-                #
+
                 if len(nameservers) > 0:
-                    #
+
                     # But we still have servers to try.  Sleep a bit
                     # so we don't pound them!
-                    #
+
                     timeout = self._compute_timeout(start)
                     sleep_time = min(timeout, backoff)
                     backoff *= 2
@@ -695,7 +710,8 @@ def query(qname, rdtype=linkcheck.dns.rdatatype.A, rdclass=linkcheck.dns.rdatacl
     object to make the query.
     @see: L{linkcheck.dns.resolver.Resolver.query} for more information on the
     parameters."""
-    linkcheck.log.debug(linkcheck.LOG_DNS, "Query %s %s %s", qname, rdtype, rdclass)
+    assert None == linkcheck.log.debug(linkcheck.LOG_DNS,
+        "Query %s %s %s", qname, rdtype, rdclass)
     if resolver is None:
         resolver = get_default_resolver()
     return resolver.query(qname, rdtype, rdclass, tcp)
@@ -727,5 +743,5 @@ def zone_for_name(name, rdclass=linkcheck.dns.rdataclass.IN,
         except (linkcheck.dns.resolver.NXDOMAIN, linkcheck.dns.resolver.NoAnswer):
             try:
                 name = name.parent()
-            except NoParent:
-                raise NoRootSoa
+            except linkcheck.dns.name.NoParent:
+                raise NoRootSOA
