@@ -18,6 +18,7 @@ import cStringIO as StringIO
 import filecmp
 import os
 import unittest
+import difflib
 
 import linkcheck.dns.exception
 import linkcheck.dns.rdata
@@ -106,28 +107,54 @@ ns1 1d1s a 10.0.0.1
 ns2 1w1D1h1m1S a 10.0.0.2
 """
 
+def get_file_lines (fn):
+    res = []
+    fd = open(fn)
+    try:
+        for line in fd:
+            res.append(line)
+    finally:
+        fd.close()
+    return res
+
+
+def get_file_diff (fn1, fn2):
+    res = []
+    c1 = get_file_lines(fn1)
+    c2 = get_file_lines(fn2)
+    for line in difflib.unified_diff(c1, c2):
+        if not isinstance(line, unicode):
+            line = unicode(line, "iso8859-1", "ignore")
+        res.append(line)
+    return res
+
+
 class TestZone (unittest.TestCase):
 
     def testFromFile1(self):
         z = linkcheck.dns.zone.from_file(fname('example'), 'example')
-        ok = False
+        diff = True
         try:
             z.to_file(fname('example1.out'), nl='\x0a')
-            ok = filecmp.cmp(fname('example1.out'), fname('example1.good'))
+            fn1 = fname('example1.out')
+            fn2 = fname('example1.good')
+            diff = get_file_diff(fn1, fn2)
         finally:
             os.unlink(fname('example1.out'))
-        self.assert_(ok)
+        self.assertFalse(diff)
 
     def testFromFile2(self):
         z = linkcheck.dns.zone.from_file(fname('example'), 'example',
                                   relativize=False)
-        ok = False
+        diff = True
         try:
             z.to_file(fname('example2.out'), relativize=False, nl='\x0a')
-            ok = filecmp.cmp(fname('example2.out'), fname('example2.good'))
+            fn1 = fname('example2.out')
+            fn2 = fname('example2.good')
+            diff = get_file_diff(fn1, fn2)
         finally:
             os.unlink(fname('example2.out'))
-        self.assert_(ok)
+        self.assertFalse(diff)
 
     def testFromText(self):
         z = linkcheck.dns.zone.from_text(example_text, 'example.', relativize=True)
@@ -303,6 +330,16 @@ class TestZone (unittest.TestCase):
         self.assertEqual(ns, [linkcheck.dns.name.from_text('ns1', None),
                                linkcheck.dns.name.from_text('ns2', None)])
 
+    def testIterateAllRdatasets(self):
+        z = linkcheck.dns.zone.from_text(example_text, 'example.', relativize=True)
+        ns = [n for n, r in z.iterate_rdatasets()]
+        ns.sort()
+        self.failUnless(ns == [linkcheck.dns.name.from_text('@', None),
+                               linkcheck.dns.name.from_text('@', None),
+                               linkcheck.dns.name.from_text('bar.foo', None),
+                               linkcheck.dns.name.from_text('ns1', None),
+                               linkcheck.dns.name.from_text('ns2', None)])
+
     def testIterateRdatas(self):
         z = linkcheck.dns.zone.from_text(example_text, 'example.', relativize=True)
         l = list(z.iterate_rdatas('A'))
@@ -317,6 +354,40 @@ class TestZone (unittest.TestCase):
                                     '10.0.0.2'))]
         self.assertEqual(l, exl)
 
+    def testIterateAllRdatas(self):
+        z = linkcheck.dns.zone.from_text(example_text, 'example.', relativize=True)
+        l = list(z.iterate_rdatas())
+        l.sort()
+        exl = [(linkcheck.dns.name.from_text('@', None),
+                3600,
+                linkcheck.dns.rdata.from_text(linkcheck.dns.rdataclass.IN,
+                                       linkcheck.dns.rdatatype.NS, 'ns1')),
+               (linkcheck.dns.name.from_text('@', None),
+                3600,
+                linkcheck.dns.rdata.from_text(linkcheck.dns.rdataclass.IN,
+                                       linkcheck.dns.rdatatype.NS, 'ns2')),
+               (linkcheck.dns.name.from_text('@', None),
+                3600,
+                linkcheck.dns.rdata.from_text(linkcheck.dns.rdataclass.IN,
+                                       linkcheck.dns.rdatatype.SOA,
+                                       'foo bar 1 2 3 4 5')),
+               (linkcheck.dns.name.from_text('bar.foo', None),
+                300,
+                linkcheck.dns.rdata.from_text(linkcheck.dns.rdataclass.IN,
+                                       linkcheck.dns.rdatatype.MX,
+                                       '0 blaz.foo')),
+               (linkcheck.dns.name.from_text('ns1', None),
+                3600,
+                linkcheck.dns.rdata.from_text(linkcheck.dns.rdataclass.IN,
+                                       linkcheck.dns.rdatatype.A,
+                                       '10.0.0.1')),
+               (linkcheck.dns.name.from_text('ns2', None),
+                3600,
+                linkcheck.dns.rdata.from_text(linkcheck.dns.rdataclass.IN,
+                                       linkcheck.dns.rdatatype.A,
+                                       '10.0.0.2'))]
+        self.assertEqual(l, exl)
+
     def testTTLs(self):
         z = linkcheck.dns.zone.from_text(ttl_example_text, 'example.', relativize=True)
         n = z['@']
@@ -352,7 +423,7 @@ class TestZone (unittest.TestCase):
         def bad():
             z = linkcheck.dns.zone.from_text(bad_directive_text, 'example.',
                                       relativize=True)
-        self.assertRaises(linkcheck.dns.exception.SyntaxError, bad)
+        self.assertRaises(linkcheck.dns.exception.DNSSyntaxError, bad)
 
 
 def test_suite ():
