@@ -16,8 +16,6 @@
 
 """DNS Zones."""
 
-from __future__ import generators
-
 import sys
 
 import linkcheck.dns.exception
@@ -404,7 +402,8 @@ class Zone(object):
             rrset = None
         return rrset
 
-    def iterate_rdatasets(self, rdtype=None, covers=linkcheck.dns.rdatatype.NONE):
+    def iterate_rdatasets(self, rdtype=linkcheck.dns.rdatatype.ANY,
+                          covers=linkcheck.dns.rdatatype.NONE):
         """Return a generator which yields (name, rdataset) tuples for
         all rdatasets in the zone which have the specified I{rdtype}
         and I{covers}.  If I{rdtype} is linkcheck.dns.rdatatype.ANY, the default,
@@ -426,7 +425,8 @@ class Zone(object):
                    (rds.rdtype == rdtype and rds.covers == covers):
                     yield (name, rds)
 
-    def iterate_rdatas(self, rdtype=None, covers=linkcheck.dns.rdatatype.NONE):
+    def iterate_rdatas(self, rdtype=linkcheck.dns.rdatatype.ANY,
+                       covers=linkcheck.dns.rdatatype.NONE):
         """Return a generator which yields (name, ttl, rdata) tuples for
         all rdatas in the zone which have the specified I{rdtype}
         and I{covers}.  If I{rdtype} is linkcheck.dns.rdatatype.ANY, the default,
@@ -581,13 +581,13 @@ class _MasterReader(object):
             name = name.relativize(self.zone.origin)
         token = self.tok.get()
         if token[0] != linkcheck.dns.tokenizer.IDENTIFIER:
-            raise linkcheck.dns.exception.SyntaxError
+            raise linkcheck.dns.exception.DNSSyntaxError
         # TTL
         try:
             ttl = linkcheck.dns.ttl.from_text(token[1])
             token = self.tok.get()
             if token[0] != linkcheck.dns.tokenizer.IDENTIFIER:
-                raise linkcheck.dns.exception.SyntaxError
+                raise linkcheck.dns.exception.DNSSyntaxError
         except linkcheck.dns.ttl.BadTTL:
             ttl = self.ttl
         # Class
@@ -595,18 +595,18 @@ class _MasterReader(object):
             rdclass = linkcheck.dns.rdataclass.from_text(token[1])
             token = self.tok.get()
             if token[0] != linkcheck.dns.tokenizer.IDENTIFIER:
-                raise linkcheck.dns.exception.SyntaxError
-        except linkcheck.dns.exception.SyntaxError:
-            raise linkcheck.dns.exception.SyntaxError
-        except:
+                raise linkcheck.dns.exception.DNSSyntaxError
+        except linkcheck.dns.exception.DNSSyntaxError:
+            raise linkcheck.dns.exception.DNSSyntaxError
+        except StandardError:
             rdclass = self.zone.rdclass
         if rdclass != self.zone.rdclass:
-            raise linkcheck.dns.exception.SyntaxError, "RR class is not zone's class"
+            raise linkcheck.dns.exception.DNSSyntaxError, "RR class is not zone's class"
         # Type
         try:
             rdtype = linkcheck.dns.rdatatype.from_text(token[1])
-        except:
-            raise linkcheck.dns.exception.SyntaxError, \
+        except StandardError:
+            raise linkcheck.dns.exception.DNSSyntaxError, \
                   "unknown rdatatype '%s'" % token[1]
         n = self.zone.nodes.get(name)
         if n is None:
@@ -615,11 +615,11 @@ class _MasterReader(object):
         try:
             rd = linkcheck.dns.rdata.from_text(rdclass, rdtype, self.tok,
                                      self.current_origin, False)
-        except linkcheck.dns.exception.SyntaxError:
+        except linkcheck.dns.exception.DNSSyntaxError:
             # Catch and reraise.
             (ty, va) = sys.exc_info()[:2]
             raise ty, va
-        except:
+        except StandardError:
             # All exceptions that occur in the processing of rdata
             # are treated as syntax errors.  This is not strictly
             # correct, but it is correct almost all of the time.
@@ -627,7 +627,7 @@ class _MasterReader(object):
             # helpful filename:line info.
 
             (ty, va) = sys.exc_info()[:2]
-            raise linkcheck.dns.exception.SyntaxError, \
+            raise linkcheck.dns.exception.DNSSyntaxError, \
                   "caught exception %s: %s" % (str(ty), str(va))
 
         rd.choose_relativity(self.zone.origin, self.relativize)
@@ -666,7 +666,7 @@ class _MasterReader(object):
                     if u == '$TTL':
                         token = self.tok.get()
                         if token[0] != linkcheck.dns.tokenizer.IDENTIFIER:
-                            raise linkcheck.dns.exception.SyntaxError, "bad $TTL"
+                            raise linkcheck.dns.exception.DNSSyntaxError, "bad $TTL"
                         self.ttl = linkcheck.dns.ttl.from_text(token[1])
                         self.tok.get_eol()
                     elif u == '$ORIGIN':
@@ -675,7 +675,7 @@ class _MasterReader(object):
                     elif u == '$INCLUDE' and self.allow_include:
                         token = self.tok.get()
                         if token[0] != linkcheck.dns.tokenizer.QUOTED_STRING:
-                            raise linkcheck.dns.exception.SyntaxError, \
+                            raise linkcheck.dns.exception.DNSSyntaxError, \
                                   "bad filename in $INCLUDE"
                         filename = token[1]
                         token = self.tok.get()
@@ -685,7 +685,7 @@ class _MasterReader(object):
                             self.tok.get_eol()
                         elif token[0] != linkcheck.dns.tokenizer.EOL and \
                              token[0] != linkcheck.dns.tokenizer.EOF:
-                            raise linkcheck.dns.exception.SyntaxError, \
+                            raise linkcheck.dns.exception.DNSSyntaxError, \
                                   "bad origin in $INCLUDE"
                         else:
                             new_origin = self.current_origin
@@ -699,16 +699,16 @@ class _MasterReader(object):
                                                            filename)
                         self.current_origin = new_origin
                     else:
-                        raise linkcheck.dns.exception.SyntaxError, \
+                        raise linkcheck.dns.exception.DNSSyntaxError, \
                               "Unknown master file directive '" + u + "'"
                     continue
                 self.tok.unget(token)
                 self._rr_line()
-        except linkcheck.dns.exception.SyntaxError, detail:
+        except linkcheck.dns.exception.DNSSyntaxError, detail:
             (filename, line_number) = self.tok.where()
             if detail is None:
                 detail = "syntax error"
-            raise linkcheck.dns.exception.SyntaxError, \
+            raise linkcheck.dns.exception.DNSSyntaxError, \
                   "%s:%d: %s" % (filename, line_number, detail)
 
         # Now that we're done reading, do some basic checking of the zone.
@@ -805,7 +805,9 @@ def from_xfr(xfr, zone_factory=Zone, relativize=True):
 
     @param xfr: The xfr generator
     @type xfr: generator of linkcheck.dns.message.Message objects
-    @param relativize: should names be relativized?  The default is True
+    @param relativize: should names be relativized?  The default is True.
+    It is essential that the relativize setting matches the one specified
+    to dns.query.xfr()
     @type relativize: bool
     @raises linkcheck.dns.zone.NoSOA: No SOA RR was found at the zone origin
     @raises linkcheck.dns.zone.NoNS: No NS RRset was found at the zone origin
