@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2005  Bastian Kleineidam
+# Copyright (C) 2000-2006 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,14 +31,13 @@ import linkcheck.url
 import linkcheck.i18n
 import linkcheck.strformat
 import linkcheck.checker
-import linkcheck.checker.cache
-import linkcheck.checker.consumer
+import linkcheck.director
 
 _logfile = None
 _supported_langs = ('de', 'fr', 'nl', 'C')
 _is_level = re.compile(r'^[0123]$').match
 
-class FormError (Exception):
+class FormError (StandardError):
     """
     Form related errors.
     """
@@ -99,13 +98,13 @@ def checklink (out=sys.stdout, form=None, env=os.environ):
     config["externlinks"].append(
              linkcheck.get_link_pat("^%s$" % linkcheck.url.safe_url_pattern))
     config["externlinks"].append(linkcheck.get_link_pat(".*", strict=True))
-    cache = linkcheck.checker.cache.Cache()
-    consumer = linkcheck.checker.consumer.Consumer(config, cache)
     # start checking
+    aggregate = linkcheck.director.get_aggregate(config)
+    get_url_from = linkcheck.checker.get_url_from
     url = form["url"].value
-    url_data = linkcheck.checker.get_url_from(url, 0, consumer, cmdline=False)
-    consumer.append_url(url_data)
-    linkcheck.checker.check_urls(consumer)
+    url_data = get_url_from(url, 0, aggregate, assume_local=False)
+    aggregate.urlqueue.put(url_data)
+    linkcheck.director.check_urls(aggregate)
 
 
 def get_host_name (form):
@@ -128,26 +127,26 @@ def checkform (form):
             os.environ['LC_MESSAGES'] = lang
             linkcheck.init_i18n()
         else:
-            raise FormError, _("unsupported language")
+            raise FormError(_("unsupported language"))
     # check url syntax
     if form.has_key("url"):
         url = form["url"].value
         if not url or url == "http://":
-            raise FormError, _("empty url was given")
+            raise FormError(_("empty url was given"))
         if not linkcheck.url.is_safe_url(url):
-            raise FormError, _("disallowed url was given")
+            raise FormError(_("disallowed url was given"))
     else:
-        raise FormError, _("no url was given")
+        raise FormError(_("no url was given"))
     # check recursion level
     if form.has_key("level"):
         level = form["level"].value
         if not _is_level(level):
-            raise FormError, _("invalid recursion level")
+            raise FormError(_("invalid recursion level"))
     # check options
     for option in ("anchors", "errors", "intern"):
         if form.has_key(option):
             if not form[option].value == "on":
-                raise FormError, _("invalid %s option syntax") % option
+                raise FormError(_("invalid %s option syntax") % option)
 
 def logit (form, env):
     """
