@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2001-2005  Bastian Kleineidam
+# Copyright (C) 2001-2006 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -77,37 +77,10 @@ class TagFinder (object):
         """
         super(TagFinder, self).__init__()
         self.content = content
-        # warnings and errors during parsing
-        self.parse_info = []
         # parser object will be initialized when it is used as
         # a handler object
         self.parser = None
 
-    def _errorfun (self, msg, name):
-        """
-        Append msg to error list.
-        """
-        self.parse_info.append(u"%s at line %d col %d: %s" % \
-            (name, self.parser.last_lineno(), self.parser.last_column(), msg))
-
-    def warning (self, msg):
-        """
-        Signal a filter/parser warning.
-        """
-        self._errorfun(msg, "warning")
-
-    def error (self, msg):
-        """
-        Signal a filter/parser error.
-        """
-        self._errorfun(msg, "error")
-
-    def fatal_error (self, msg):
-        """
-        Signal a fatal filter/parser error.
-        """
-        self._errorfun(msg, "fatal error")
-
     def start_element (self, tag, attrs):
         """
         Does nothing, override in a subclass.
@@ -134,7 +107,8 @@ class MetaRobotsFinder (TagFinder):
         super(MetaRobotsFinder, self).__init__(content)
         self.follow = True
         self.index = True
-        linkcheck.log.debug(linkcheck.LOG_CHECK, "meta robots finder")
+        assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+            "meta robots finder")
 
     def start_element (self, tag, attrs):
         """
@@ -180,18 +154,18 @@ class LinkFinder (TagFinder):
             self.tags = tags
         self.urls = []
         self.base_ref = u''
-        linkcheck.log.debug(linkcheck.LOG_CHECK, "link finder")
+        assert None == linkcheck.log.debug(linkcheck.LOG_CHECK, "link finder")
 
     def start_element (self, tag, attrs):
         """
         Search for links and store found URLs in a list.
         """
-        linkcheck.log.debug(linkcheck.LOG_CHECK, "LinkFinder tag %s attrs %s",
-                            tag, attrs)
-        linkcheck.log.debug(linkcheck.LOG_CHECK,
-                            "line %d col %d old line %d old col %d",
-                            self.parser.lineno(), self.parser.column(),
-                         self.parser.last_lineno(), self.parser.last_column())
+        assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+            "LinkFinder tag %s attrs %s", tag, attrs)
+        assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+            "line %d col %d old line %d old col %d",
+            self.parser.lineno(), self.parser.column(),
+            self.parser.last_lineno(), self.parser.last_column())
         if tag == "base" and not self.base_ref:
             self.base_ref = attrs.get_true("href", u'')
         tagattrs = self.tags.get(tag, [])
@@ -214,8 +188,8 @@ class LinkFinder (TagFinder):
             value = unquote(attrs.get(attr))
             # add link to url list
             self.add_link(tag, attr, value, name, codebase)
-        linkcheck.log.debug(linkcheck.LOG_CHECK,
-                            "LinkFinder finished tag %s", tag)
+        assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+            "LinkFinder finished tag %s", tag)
 
     def get_link_name (self, tag, attrs, attr):
         """
@@ -249,13 +223,13 @@ class LinkFinder (TagFinder):
         assert isinstance(url, unicode) or url is None, repr(url)
         urls = []
         # look for meta refresh
-        if tag == u'meta':
+        if tag == u'meta' and url:
             mo = refresh_re.match(url)
             if mo:
                 urls.append(mo.group("url"))
             elif attr != 'content':
                 urls.append(url)
-        elif attr == u'style':
+        elif attr == u'style' and url:
             for mo in css_url_re.finditer(url):
                 u = mo.group("url")
                 urls.append(unquote(u, matching=True))
@@ -266,7 +240,7 @@ class LinkFinder (TagFinder):
             return
         for u in urls:
             assert isinstance(u, unicode) or u is None, repr(u)
-            linkcheck.log.debug(linkcheck.LOG_CHECK,
+            assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
               u"LinkParser add link %s %s %s %s %s", tag, attr, u, name, base)
             self.urls.append((u, self.parser.last_lineno(),
                               self.parser.last_column(), name, base))
