@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2005  Bastian Kleineidam
+# Copyright (C) 2000-2006 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -88,7 +88,7 @@ class BlacklistLogger (linkcheck.logger.Logger):
         Write the blacklist.
         """
         oldmask = os.umask(0077)
-        for key, value in self.blacklist.items():
+        for key, value in self.blacklist.iteritems():
             self.fd.write("%d %s" % (value, key))
             self.fd.write(os.linesep)
         self.close_fileoutput()
