@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2005  Bastian Kleineidam
+# Copyright (C) 2000-2006 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -70,7 +70,7 @@ class CustomXMLLogger (linkcheck.logger.xmllog.XMLLogger):
             self.xml_tag(u"checktime", u"%f" % url_data.checktime)
         if url_data.info and self.has_part('info'):
             self.xml_starttag(u"infos")
-            for info in url_data.info:
+            for tag, info in url_data.info:
                 self.xml_tag(u"info", info)
             self.xml_endtag(u"infos")
         if url_data.warnings and self.has_part('warning'):
