@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2005  Bastian Kleineidam
+# Copyright (C) 2000-2006 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -47,12 +47,12 @@ class GMLLogger (linkcheck.logger.Logger):
             return
         self.starttime = time.time()
         if self.has_part("intro"):
-            self.comment(_("created by %s at %s") % \
+            self.comment(_("created by %s at %s") %
                          (linkcheck.configuration.AppName,
                           linkcheck.strformat.strtime(self.starttime)))
-            self.comment(_("Get the newest version at %(url)s") % \
+            self.comment(_("Get the newest version at %(url)s") %
                          {'url': linkcheck.configuration.Url})
-            self.comment(_("Write comments and bugs to %(email)s") % \
+            self.comment(_("Write comments and bugs to %(email)s") %
                          {'email': linkcheck.configuration.Email})
             self.check_date()
             self.writeln()
@@ -97,12 +97,12 @@ class GMLLogger (linkcheck.logger.Logger):
         Write all edges we can find in the graph in a brute-force
         manner. Better would be a mapping of parent URLs.
         """
-        for node in self.nodes.values():
+        for node in self.nodes.itervalues():
             if self.nodes.has_key(node.parent_url):
                 self.writeln(u"  edge [")
                 self.writeln(u'    label  "%s"' % (node.base_url or u""))
                 if self.has_part("parenturl") and node.parent_url:
-                    self.writeln(u"    source %d" % \
+                    self.writeln(u"    source %d" %
                                  self.nodes[node.parent_url].id)
                 self.writeln(u"    target %d" % node.id)
                 if self.has_part("result"):
@@ -121,7 +121,7 @@ class GMLLogger (linkcheck.logger.Logger):
         if self.has_part("outro"):
             self.stoptime = time.time()
             duration = self.stoptime - self.starttime
-            self.comment(_("Stopped checking at %s (%s)")%\
+            self.comment(_("Stopped checking at %s (%s)") %
                          (linkcheck.strformat.strtime(self.stoptime),
-                          linkcheck.strformat.strduration(duration)))
+                          linkcheck.strformat.strduration_long(duration)))
         self.close_fileoutput()
