@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2005  Bastian Kleineidam
+# Copyright (C) 2000-2006 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -79,7 +79,7 @@ class GraphXMLLogger (linkcheck.logger.xmllog.XMLLogger):
         Write all edges we can find in the graph in a brute-force
         manner. Better would be a mapping of parent URLs.
         """
-        for node in self.nodes.values():
+        for node in self.nodes.itervalues():
             if self.nodes.has_key(node.parent_url):
                 attrs = {
                     u"source": u"%d" % self.nodes[node.parent_url].id,
