@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2005  Bastian Kleineidam
+# Copyright (C) 2000-2006 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -78,12 +78,12 @@ class SQLLogger (linkcheck.logger.Logger):
             return
         self.starttime = time.time()
         if self.has_part("intro"):
-            self.comment(_("created by %s at %s") % \
+            self.comment(_("created by %s at %s") %
                          (linkcheck.configuration.AppName,
                           linkcheck.strformat.strtime(self.starttime)))
-            self.comment(_("Get the newest version at %s") % \
+            self.comment(_("Get the newest version at %s") %
                          linkcheck.configuration.Url)
-            self.comment(_("Write comments and bugs to %s") % \
+            self.comment(_("Write comments and bugs to %s") %
                          linkcheck.configuration.Email)
             self.check_date()
             self.writeln()
@@ -96,6 +96,7 @@ class SQLLogger (linkcheck.logger.Logger):
         if self.fd is None:
             return
         log_warnings = [x[1] for x in url_data.warnings]
+        log_infos = [x[1] for x in url_data.info]
         self.writeln(u"insert into %(table)s(urlname,recursionlevel,"
               "parentname,baseref,valid,result,warning,info,url,line,col,"
               "name,checktime,dltime,dlsize,cached) values ("
@@ -115,7 +116,7 @@ class SQLLogger (linkcheck.logger.Logger):
               "%(dltime)d,"
               "%(dlsize)d,"
               "%(cached)d"
-              ")%(separator)s" % \
+              ")%(separator)s" %
               {'table': self.dbname,
                'base_url': sqlify(url_data.base_url or u""),
                'recursion_level': url_data.recursion_level,
@@ -124,7 +125,7 @@ class SQLLogger (linkcheck.logger.Logger):
                'valid': intify(url_data.valid),
                'result': sqlify(url_data.result),
                'warning': sqlify(os.linesep.join(log_warnings)),
-               'info': sqlify(os.linesep.join(url_data.info)),
+               'info': sqlify(os.linesep.join(log_infos)),
                'url': sqlify(linkcheck.url.url_quote(url_data.url or u"")),
                'line': url_data.line,
                'column': url_data.column,
@@ -146,7 +147,7 @@ class SQLLogger (linkcheck.logger.Logger):
         if self.has_part("outro"):
             self.stoptime = time.time()
             duration = self.stoptime - self.starttime
-            self.comment(_("Stopped checking at %s (%s)") % \
+            self.comment(_("Stopped checking at %s (%s)") %
                          (linkcheck.strformat.strtime(self.stoptime),
-                          linkcheck.strformat.strduration(duration)))
+                          linkcheck.strformat.strduration_long(duration)))
         self.close_fileoutput()
