@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2005  Bastian Kleineidam
+# Copyright (C) 2000-2006 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -206,15 +206,16 @@ class TextLogger (linkcheck.logger.Logger):
         Write url_data.info.
         """
         self.write(self.part("info") + self.spaces("info"))
-        self.writeln(self.wrap(url_data.info, 65), color=self.colorinfo)
+        text = [x[1] for x in url_data.info]
+        self.writeln(self.wrap(text, 65), color=self.colorinfo)
 
     def write_warning (self, url_data):
         """
         Write url_data.warning.
         """
         self.write(self.part("warning") + self.spaces("warning"))
-        log_warnings = [x[1] for x in url_data.warnings]
-        self.writeln(self.wrap(log_warnings, 65), color=self.colorwarning)
+        text = [x[1] for x in url_data.warnings]
+        self.writeln(self.wrap(text, 65), color=self.colorwarning)
 
     def write_result (self, url_data):
         """
@@ -228,7 +229,7 @@ class TextLogger (linkcheck.logger.Logger):
             color = self.colorinvalid
             self.write(_("Error"), color=color)
         if url_data.result:
-            self.write(u": "+url_data.result, color=color)
+            self.write(u": " + url_data.result, color=color)
         self.writeln()
 
     def end_output (self):
@@ -239,7 +240,7 @@ class TextLogger (linkcheck.logger.Logger):
             return
         if self.has_part('outro'):
             self.writeln()
-            self.write(_("That's it.")+" ")
+            self.write(_("That's it.") + " ")
             if self.number >= 0:
                 self.write(_n("%d link checked.", "%d links checked.",
                               self.number) % self.number)
@@ -252,7 +253,7 @@ class TextLogger (linkcheck.logger.Logger):
                             self.errors) % self.errors)
             self.stoptime = time.time()
             duration = self.stoptime - self.starttime
-            self.writeln(_("Stopped checking at %s (%s)") % \
+            self.writeln(_("Stopped checking at %s (%s)") %
                          (linkcheck.strformat.strtime(self.stoptime),
-                          linkcheck.strformat.strduration(duration)))
+                          linkcheck.strformat.strduration_long(duration)))
         self.close_fileoutput()
