@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2005  Bastian Kleineidam
+# Copyright (C) 2000-2006 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,32 +28,16 @@ import urllib2
 import time
 import socket
 import re
-import sys
 import zlib
 import gzip
+import sys
 import cStringIO as StringIO
 import linkcheck
 import linkcheck.configuration
+import linkcheck.log
 
 __all__ = ["RobotFileParser"]
 
-_debug = False
-
-def _msg (prefix, msg):
-    """
-    Print given prefix and debug message to stderr if the _debug flag is
-    set.
-
-    @return: None
-    """
-    if _debug:
-        print >> sys.stderr, prefix, msg
-
-# methods for debug, warning and error messages
-debug = lambda txt: _msg("debug:", txt)
-warn = lambda txt: _msg("warning:", txt)
-error = lambda txt: _msg("error:", txt)
-
 
 class PasswordManager (object):
     """
@@ -181,45 +165,66 @@ class RobotFileParser (object):
         }
         req = urllib2.Request(self.url, None, headers)
         try:
-            f = self.get_opener().open(req)
+            self._read_content(req)
         except urllib2.HTTPError, x:
             if x.code in (401, 403):
                 self.disallow_all = True
-                debug("robotst.txt disallow all")
+                assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                    "%s disallow all", self.url)
             else:
                 self.allow_all = True
-                debug("robots.txt allow all")
-            return
-        except (socket.gaierror, socket.error, urllib2.URLError), x:
+                assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                    "%s allow all", self.url)
+        except socket.timeout:
+            raise
+        except urllib2.URLError:
+            x = sys.exc_info()[1]
+            if isinstance(x.reason, socket.timeout):
+                raise
+            self.allow_all = True
+            assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                "%s allow all", self.url)
+        except (socket.gaierror, socket.error):
             # no network
             self.allow_all = True
-            debug("robots.txt allow all")
-            return
-        except IOError, data:
-            if data and data[0] == 'http error' and data[1] == 404:
-                self.allow_all = True
-                debug("robots.txt allow all")
-            else:
-                self.allow_all = True
-                debug("robots.txt allow all")
-            return
+            assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                "%s allow all", self.url)
+        except IOError, msg:
+            self.allow_all = True
+            assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                "%s allow all", self.url)
         except httplib.HTTPException:
             self.allow_all = True
-            debug("robots.txt allow all")
-            return
+            assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                "%s allow all", self.url)
         except ValueError:
             # XXX bug workaround:
             # urllib2.AbstractDigestAuthHandler raises ValueError on
             # failed authorisation
             self.disallow_all = True
-            debug("robotst.txt disallow all")
-            return
-        lines = []
-        line = f.readline()
-        while line:
-            lines.append(line.strip())
+            assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                "%s disallow all", self.url)
+
+    def _read_content (self, req):
+        """
+        Read robots.txt content.
+        @raise: urllib2.HTTPError on HTTP failure codes
+        @raise: socket.gaierror, socket.error, urllib2.URLError on network
+          errors
+        @raise: httplib.HTTPException, IOError on HTTP errors
+        @raise: ValueError on bad digest auth (a bug)
+        """
+        f = self.get_opener().open(req)
+        ct = f.info().get("Content-Type")
+        if ct and ct.lower().startswith("text/plain"):
+            lines = []
             line = f.readline()
-        self.parse(lines)
+            while line:
+                lines.append(line.strip())
+                line = f.readline()
+            self.parse(lines)
+        else:
+            self.allow_all = True
 
     def _add_entry (self, entry):
         """
@@ -241,7 +246,8 @@ class RobotFileParser (object):
 
         @return: None
         """
-        debug("robots.txt parse lines")
+        assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+            "%s parse lines", self.url)
         state = 0
         linenumber = 0
         entry = Entry()
@@ -250,9 +256,9 @@ class RobotFileParser (object):
             linenumber += 1
             if not line:
                 if state == 1:
-                    warn("line %d: you should insert"
-                          " allow: or disallow: directives below any"
-                          " user-agent: line" % linenumber)
+                    assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                         "%s line %d: allow or disallow directives without" \
+                         " any user-agent line", self.url, linenumber)
                     entry = Entry()
                     state = 0
                 elif state == 2:
@@ -272,33 +278,54 @@ class RobotFileParser (object):
                 line[1] = urllib.unquote(line[1].strip())
                 if line[0] == "user-agent":
                     if state == 2:
-                        warn("line %d: you should insert a blank"
-                             " line before any user-agent"
-                             " directive" % linenumber)
+                        assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                          "%s line %d: missing blank line before user-agent" \
+                          " directive", self.url, linenumber)
                         self._add_entry(entry)
                         entry = Entry()
                     entry.useragents.append(line[1])
                     state = 1
                 elif line[0] == "disallow":
                     if state == 0:
-                        error("line %d: you must insert a user-agent:"
-                              " directive before this line" % linenumber)
+                        assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                          "%s line %d: missing user-agent directive before" \
+                          " this line", self.url, linenumber)
                     else:
                         entry.rulelines.append(RuleLine(line[1], 0))
                         state = 2
                 elif line[0] == "allow":
                     if state == 0:
-                        error("line %d: you must insert a user-agent:"
-                               " directive before this line" % linenumber)
+                        assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                          "%s line %d: missing user-agent directive before" \
+                          " this line", self.url, linenumber)
                     else:
                         entry.rulelines.append(RuleLine(line[1], 1))
+                elif line[0] == "crawl-delay":
+                    if state == 0:
+                        assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                          "%s line %d: missing user-agent directive before" \
+                          " this line", self.url, linenumber)
+                    else:
+                        try:
+                            entry.crawldelay = max(0, int(line[1]))
+                            state = 2
+                        except ValueError:
+                            assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                             "%s line %d: invalid delay number %r",
+                             self.url, linenumber, line[1])
+                            pass
                 else:
-                    warn("line %d: unknown key %s" % (linenumber, line[0]))
+                    assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                             "%s line %d: unknown key %s",
+                             self.url, linenumber, line[0])
             else:
-                error("line %d: malformed line %s" % (linenumber, line))
-        if state == 2:
+                assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                    "%s line %d: malformed line %s",
+                    self.url, linenumber, line)
+        if state in (1, 2):
             self.entries.append(entry)
-        debug("Parsed rules:\n%s" % str(self))
+        assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                            "Parsed rules:\n%s", str(self))
 
     def can_fetch (self, useragent, url):
         """
@@ -307,8 +334,9 @@ class RobotFileParser (object):
         @return: True if agent can fetch url, else False
         @rtype: bool
         """
-        debug("Checking robot.txt allowance for:\n"\
-              "  user agent: %r\n  url: %r" % (useragent, url))
+        assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+              "%s check allowance for:\n" \
+              "  user agent: %r\n  url: %r", self.url, useragent, url)
         if not isinstance(useragent, str):
             useragent = useragent.encode("ascii", "ignore")
         if not isinstance(url, str):
@@ -329,6 +357,18 @@ class RobotFileParser (object):
         # agent not found ==> access granted
         return True
 
+    def get_crawldelay (self, useragent):
+        """
+        Look for a configured crawl delay.
+
+        @return: crawl delay in seconds or zero
+        @rtype: integer >= 0
+        """
+        for entry in self.entries:
+            if entry.applies_to(useragent):
+                return entry.crawldelay
+        return 0
+
     def __str__ (self):
         """
         Constructs string representation, usable as contents of a
@@ -389,6 +429,7 @@ class Entry (object):
         """
         self.useragents = []
         self.rulelines = []
+        self.crawldelay = 0
 
     def __str__ (self):
         """
@@ -397,7 +438,9 @@ class Entry (object):
         @return: robots.txt format
         @rtype: string
         """
-        lines = ["User-agent: %r" % agent for agent in self.useragents]
+        lines = ["User-agent: %s" % agent for agent in self.useragents]
+        if self.crawldelay:
+            lines.append("Crawl-delay: %d" % self.crawldelay)
         lines.extend([str(line) for line in self.rulelines])
         return "\n".join(lines)
 
@@ -433,7 +476,8 @@ class Entry (object):
         @rtype: bool
         """
         for line in self.rulelines:
-            debug("%s %s %s" % (filename, str(line), line.allowance))
+            assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+               "%s %s %s", filename, str(line), line.allowance)
             if line.applies_to(filename):
                 return line.allowance
         return True
@@ -463,7 +507,8 @@ def decode (page):
     """
     Gunzip or deflate a compressed page.
     """
-    debug("robots.txt page info %s" % str(page.info()))
+    assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+      "robots.txt page info %d %s", page.code, str(page.info()))
     encoding = page.info().get("Content-Encoding")
     if encoding in ('gzip', 'x-gzip', 'deflate'):
         # cannot seek in socket descriptors, so must get content now
@@ -474,8 +519,9 @@ def decode (page):
             else:
                 fp = gzip.GzipFile('', 'rb', 9, StringIO.StringIO(content))
         except zlib.error, msg:
-            warn("uncompressing had error "\
-                 "%s, assuming non-compressed content" % str(msg))
+            assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                 "uncompressing had error "
+                 "%s, assuming non-compressed content", str(msg))
             fp = StringIO.StringIO(content)
         # remove content-encoding header
         headers = httplib.HTTPMessage(StringIO.StringIO(""))
