@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2004-2005  Bastian Kleineidam
+# Copyright (C) 2004-2006 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +19,6 @@ Test robots.txt parsing.
 """
 
 import unittest
-
 import linkcheck.robotparser2
 
 
@@ -34,8 +33,6 @@ class TestRobotParser (unittest.TestCase):
         """
         Initialize self.rp as a robots.txt parser.
         """
-        if hasattr(self, "needed_resources"):
-            self.check_resources(self.needed_resources)
         self.rp = linkcheck.robotparser2.RobotFileParser()
 
     def check (self, a, b):
