@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2004-2005  Bastian Kleineidam
+# Copyright (C) 2004-2006 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@ Test url routines.
 
 import unittest
 import os
+import re
 import linkcheck.url
 
 # 'ftp://user:pass@ftp.foo.net/foo/bar':
@@ -45,6 +46,11 @@ class TestUrl (unittest.TestCase):
     Test url norming and quoting.
     """
 
+    def urlnormtest (self, url, nurl):
+        self.assertEquals(url_norm(url), nurl)
+        cs = "iso8859-1"
+        self.assertEquals(url_norm(url.decode(cs)), nurl.decode(cs))
+
     def test_pathattack (self):
         """
         Windows winamp path attack prevention.
@@ -54,47 +60,91 @@ class TestUrl (unittest.TestCase):
         nurl = "http://server/cskin.zip"
         self.assertEquals(linkcheck.url.url_quote(url_norm(url)), nurl)
 
+    def test_stripsite (self):
+        stripsite = linkcheck.url.stripsite
+        url = "http://imadoofus.org/a/b/c"
+        self.assertEqual(stripsite(url)[0], "imadoofus.org")
+        self.assertEqual(stripsite(url)[1], "/a/b/c")
+
+    def test_safe_patterns (self):
+        is_safe_host = linkcheck.url.is_safe_host
+        safe_host_pattern = linkcheck.url.safe_host_pattern
+        self.assert_(is_safe_host("imadoofus.org"))
+        self.assert_(is_safe_host("imadoofus.org:80"))
+        self.assert_(not is_safe_host("imadoofus.org:21"))
+        pat = safe_host_pattern("imadoofus.org")
+        ro = re.compile(pat)
+        self.assert_(ro.match("http://imadoofus.org:80/"))
+
+    def test_url_quote (self):
+        url_quote = linkcheck.url.url_quote
+        url = "http://a:80/bcd"
+        self.assertEquals(url_quote(url), url)
+        url = "http://a:80/bcd?"
+        url2 = "http://a:80/bcd"
+        self.assertEquals(url_quote(url), url2)
+        url = "http://a:80/bcd?a=b"
+        url2 = "http://a:80/bcd?a=b"
+        self.assertEquals(url_quote(url), url2)
+        url = "a/b"
+        self.assertEquals(url_quote(url), url)
+        url = "bcd?"
+        url2 = "bcd"
+        self.assertEquals(url_quote(url), url2)
+        url = "bcd?a=b"
+        url2 = "bcd?a=b"
+        self.assertEquals(url_quote(url), url2)
+
+
     def test_norm_quote (self):
         """
         Test url norm quoting.
         """
         url = "http://groups.google.com/groups?hl=en&lr&ie=UTF-8&"\
               "threadm=3845B54D.E546F9BD%40monmouth.com&rnum=2&"\
-              "prev=/groups%3Fq%3Dlogitech%2Bwingman%2Bextreme%2Bdigital"\
+              "prev=%2Fgroups%3Fq%3Dlogitech%2Bwingman%2Bextreme%2Bdigital"\
               "%2B3d%26hl%3Den%26lr%3D%26ie%3DUTF-8%26selm%3D3845B54D.E5"\
               "46F9BD%2540monmouth.com%26rnum%3D2"
-        nurl = url
-        self.assertEqual(url_norm(url), nurl)
+        self.urlnormtest(url, url)
         url = "http://redirect.alexa.com/redirect?"\
               "http://www.offeroptimizer.com"
         nurl = url
-        self.assertEqual(url_norm(url), nurl)
+        self.urlnormtest(url, nurl)
         url = "http://www.lesgensducinema.com/photo/Philippe%20Nahon.jpg"
         nurl = url
-        self.assertEqual(url_norm(url), nurl)
+        self.urlnormtest(url, nurl)
         # Only perform percent-encoding where it is essential.
         url = "http://example.com/%7Ejane"
         nurl = "http://example.com/~jane"
-        self.assertEqual(url_norm(url), nurl)
+        self.urlnormtest(url, nurl)
         url = "http://example.com/%7ejane"
-        self.assertEqual(url_norm(url), nurl)
+        self.urlnormtest(url, nurl)
         # Always use uppercase A-through-F characters when percent-encoding.
         url = "http://example.com/?q=1%2a2"
         nurl = "http://example.com/?q=1%2A2"
-        self.assertEqual(url_norm(url), nurl)
+        self.urlnormtest(url, nurl)
         # the no-quote chars
         url = "http://example.com/a*+-();b"
         nurl = url
-        self.assertEqual(url_norm(url), nurl)
-        url = "http://www.company.com/path/doc.html?url=/path2/doc2.html?foo=bar"
+        self.urlnormtest(url, nurl)
+        url = "http://www.company.com/path/doc.html?url=%2Fpath2%2Fdoc2.html?foo=bar"
         nurl = url
-        self.assertEqual(url_norm(url), nurl)
+        self.urlnormtest(url, nurl)
         url = "http://example.com/#a b"
         nurl = "http://example.com/#a%20b"
-        self.assertEqual(url_norm(url), nurl)
+        self.urlnormtest(url, nurl)
         url = "http://example.com/?u=http://example2.com?b=c "
-        nurl ="http://example.com/?u=http://example2.com?b=c%20"
-        self.assertEqual(url_norm(url), nurl)
+        nurl ="http://example.com/?u=http:%2F%2Fexample2.com?b=c+"
+        self.urlnormtest(url, nurl)
+        url = "http://example.com/?u=http://example2.com?b="
+        nurl ="http://example.com/?u=http:%2F%2Fexample2.com?b="
+        self.urlnormtest(url, nurl)
+        url = "http://localhost:8001/?quoted="
+        nurl = "http://localhost:8001/?quoted=%FC"
+        self.urlnormtest(url, nurl)
+        url = "http://host/?a=b/c+d="
+        nurl = "http://host/?a=b%2Fc+d%3D"
+        self.urlnormtest(url, nurl)
 
     def test_norm_case_sensitivity (self):
         """
@@ -367,6 +417,16 @@ class TestUrl (unittest.TestCase):
         nurl = url
         self.assertEqual(url_norm(url), nurl)
 
+    def test_norm_file1 (self):
+        url = "file:///a/b.txt"
+        nurl = url
+        self.assertEqual(url_norm(url), nurl)
+
+    def test_norm_file2 (self):
+        url = "file://C|/a/b.txt"
+        nurl = "file://c%7C/a/b.txt"
+        self.assertEqual(url_norm(url), nurl)
+
     def test_norm_invalid (self):
         url = u"?:"
         nurl = u"%E4%F6%FC?:"
@@ -382,6 +442,9 @@ class TestUrl (unittest.TestCase):
         url = u"http//www.imadoofus.org"
         nurl = u"http://www.imadoofus.org"
         self.assertEqual(linkcheck.url.url_fix_common_typos(url), nurl)
+        url = u"https//www.imadoofus.org"
+        nurl = u"https://www.imadoofus.org"
+        self.assertEqual(linkcheck.url.url_fix_common_typos(url), nurl)
 
     def test_valid (self):
         """
@@ -397,10 +460,6 @@ class TestUrl (unittest.TestCase):
         self.assert_(linkcheck.url.is_safe_url(u), u)
         u = "http://www.imadoofus.com#anchor55"
         self.assert_(linkcheck.url.is_safe_url(u), u)
-        u = "http://www.imadoofus.com/?hulla=do"
-        self.assert_(linkcheck.url.is_safe_js_url(u), u)
-        u = "http://www.imadoofus.com/foo.bar/woot/bla;a=120x600;b=615660"
-        self.assert_(linkcheck.url.is_safe_js_url(u), u)
 
     def test_needs_quoting (self):
         """
@@ -446,20 +505,32 @@ class TestUrl (unittest.TestCase):
         Test idna encoding.
         """
         url = u'www.ko.de'
-        encurl, is_idn = linkcheck.url.idna_encode(url)
+        idna_encode =linkcheck.url.idna_encode
+        encurl, is_idn = idna_encode(url)
         self.assert_(is_idn)
         url = u''
-        encurl, is_idn = linkcheck.url.idna_encode(url)
+        encurl, is_idn = idna_encode(url)
         self.assert_(not is_idn)
         self.assert_(not encurl)
+        url = u"http://www.imadoofus.org%0D%3Cfont%20face=%22Verdana,%20Arial,%20Helvetica,%20sans-serif%22%20size=%222%22%3E%3Chttp://www.imadoofus.org%3E%20%0D%20%20%20%20%20%20%20%20%20%20%20%20%20%20"
+        self.assertRaises(UnicodeError, idna_encode, url)
 
     def test_match_host (self):
         """
         Test host matching.
         """
-        self.assert_(not linkcheck.url.match_host("localhost", [".localhost"]))
-        self.assert_(linkcheck.url.match_host("a.localhost", [".localhost"]))
-        self.assert_(linkcheck.url.match_host("localhost", ["localhost"]))
+        match_host = linkcheck.url.match_host
+        match_url = linkcheck.url.match_url
+        self.assert_(not match_host("", []))
+        self.assert_(not match_host("", [".localhost"]))
+        self.assert_(not match_host("localhost", []))
+        self.assert_(not match_host("localhost", [".localhost"]))
+        self.assert_(match_host("a.localhost", [".localhost"]))
+        self.assert_(match_host("localhost", ["localhost"]))
+        self.assert_(not match_url("", []))
+        self.assert_(not match_url("a", []))
+        self.assert_(match_url("http://imadoofus.org/hulla",
+                               ["imadoofus.org"]))
 
     def test_splitparam (self):
         """
@@ -487,6 +558,23 @@ class TestUrl (unittest.TestCase):
         u = "scid=kb;en-us;Q248840&b=c;hulla=bulla"
         self.assertEqual(linkcheck.url.url_parse_query(u), u)
 
+    def test_port (self):
+        is_numeric_port = linkcheck.url.is_numeric_port
+        self.assert_(is_numeric_port("80"))
+        self.assert_(is_numeric_port("1"))
+        self.assertFalse(is_numeric_port("0"))
+        self.assertFalse(is_numeric_port("66000"))
+        self.assertFalse(is_numeric_port("-1"))
+        self.assertFalse(is_numeric_port("a"))
+
+    def test_split (self):
+        url_split = linkcheck.url.url_split
+        url_unsplit = linkcheck.url.url_unsplit
+        url = "http://imadoofus.org/whoops"
+        self.assertEqual(url_unsplit(url_split(url)), url)
+        url = "http://imadoofus.org:123/whoops"
+        self.assertEqual(url_unsplit(url_split(url)), url)
+
 
 def test_suite ():
     """
